@@ -29,7 +29,7 @@
 
 import os
 import re
-from unilint.cpp_source_plugin import AbstractCppPlugin, CPP_EXTENSIONS
+from unilint.cpp_source_plugin import AbstractCppPlugin
 from unilint.unilint_plugin import UnilintPluginInitException
 from unilint.unilint_main import LEVEL_WARNING, LEVEL_STYLE, LEVELS
 from unilint.issue import Issue
@@ -88,7 +88,6 @@ class CppcheckPlugin(AbstractCppPlugin):
         _, output, _ = self.shell_cmd(cmd,
                                       shell=True,
                                       us_env=True,
-                                      cwd=os.path.dirname(path),
                                       capture_std_err=True)
 
         if options.raw:
