@@ -80,10 +80,11 @@ class PylintPlugin(AbstractPythonPlugin):
         cmd = 'pylint %s --output-format=parseable -i y -r n %s %s' % (path, lvl_cmd, ' '.join(suppress_options))
         if options.verbose:
             print(cmd)
+
         _, output, _ = self.shell_cmd(cmd,
                                       shell=True,
                                       us_env=True,
-                                      cwd=os.path.dirname(path),
+                                      cwd=os.path.dirname(os.path.abspath(path)),
                                       ignore_returncodes=range(50))
         output += '\n'
         basepath = os.path.dirname(path)
