@@ -23,10 +23,16 @@ LEVELS = {LEVEL_ALL: 0, LEVEL_STYLE: 10, LEVEL_WARNING: 20, LEVEL_ERROR: 30}
 
 
 def register_formatter(f_id, formatter_function):
+    """
+    Add a formatter with an id so that users may select it
+    """
     FORMATTERS[f_id] = formatter_function
 
 
 def register_plugin(plugin):
+    """
+    Add a static code analyzer plugin
+    """
     pid = plugin.get_id()
     if pid is None or pid == '':
         raise UnilintException('Invalid plugin id %s' % (pid))
@@ -110,24 +116,30 @@ def remove_duplicates(issues):
     result = []
     sameline_issues = []
     lastline = -1
+    lastcolumn = -1
     for issue in issues2:
-        if issue.line_number_start != lastline:
+        if (issue.line_number_start != lastline) or \
+                (issue.line_position >= 0 and
+                 lastcolumn >= 0 and
+                 issue.line_position != lastcolumn):
             sameline_issues = []
             lastline = issue.line_number_start
+            lastcolumn = issue.line_position
         else:
             is_duplicate = False
             for oldissue in sameline_issues:
                 # for now, will rely on manually crafted duplicate
-                # detection
-                for keyword_vector in [('too long'),
-                                       ('indentation contains'),
-                                       ('variable '),
+                # detection for similar error in different code analysers
+                for keyword_vector in [('too long', ),
+                                       ('indentation contains', ),
+                                       ('variable ', ),
                                        ('not used', 'unused')]:
-                    if keyword_vector[0] in oldissue.message:
-                        for keyword_match in keyword_vector:
-                            if keyword_match in issue.message:
-                                is_duplicate = True
-                                break
+                    for keyword_match_old in keyword_vector:
+                        if keyword_match_old in oldissue.message:
+                            for keyword_match in keyword_vector:
+                                if keyword_match in issue.message:
+                                    is_duplicate = True
+                                    break
                     if is_duplicate:
                         break
             if is_duplicate:
@@ -184,14 +196,13 @@ def check_path(path_to_check,
     if not os.path.exists(path_to_check):
         print('Ignoring argument that does not exist: %s' % path_to_check)
         return
-    issues = []
     categorized_resources = {}
-    for plugin_id in args.selected_plugins:
+    for plugin in args.selected_plugins.values():
         new_categories = \
-            args.selected_plugins[plugin_id].categorize_type(args,
-                                                             path_to_check,
-                                                             subdirs,
-                                                             files) or {}
+            plugin.categorize_type(args,
+                                   path_to_check,
+                                   subdirs,
+                                   files) or {}
 
         for resource, categories in new_categories.items():
             # ignore folder ?
@@ -206,23 +217,34 @@ def check_path(path_to_check,
                     if not args.check_docs:
                         raise IgnorableFolderException()
         extend_maybe(categorized_resources, new_categories)
-
-    for resource, categories in categorized_resources.items():
+    issues = []
+    for resource, categories in sorted(categorized_resources.items()):
+        resource_issues = []
         if 'hidden' in categories:
             continue
         if 'backup' in categories:
             continue
         if args.debug:
             print("processing: %s as %s" % (resource, categories))
-        for plugin_id in args.selected_plugins:
-            result = args.selected_plugins[plugin_id].check_resource(args,
-                                                                     resource,
-                                                                     categories)
-            extend_maybe(issues, result)
-            if result is not None and not args.ordered and not args.raw:
-                # get the output to the user as quickly as possible
-                print_issues(args, result, basepath)
-    return remove_duplicates(issues)
+        for plugin in args.selected_plugins.values():
+            try:
+                result = plugin.check_resource(args,
+                                               resource,
+                                               categories)
+                extend_maybe(resource_issues, result)
+                if result is not None and not args.ordered and not args.raw:
+                    # get the output to the user as quickly as possible
+                    print_issues(args, result, basepath)
+            except KeyboardInterrupt as kbi:
+                raise UnilintException('Interrupted while running %s on %s: %s' % (plugin.get_id(), resource, kbi))
+        if resource_issues:
+            resource_issues = remove_duplicates(resource_issues)
+            extend_maybe(issues, resource_issues)
+            if args.ordered and not args.raw:
+                # print here unless we already printed
+                resource_issues = order_issues(resource_issues)
+                print_issues(args, resource_issues, basepath)
+    return issues
 
 
 def run_cmd(paths, args):
@@ -287,10 +309,6 @@ def run_cmd(paths, args):
                                             force=os.path.samefile(parentdir,
                                                                    path))
                     extend_maybe(issues, new_issues)
-                    if not args.raw and args.ordered:
-                        # print here unless we already printed in check_path
-                        dir_issues = order_issues(new_issues)
-                        print_issues(args, dir_issues, abs_path)
                 except IgnorableFolderException:
                     # beware concurrent modification, requires [:]
                     ignoreddirs = subdirs[:]
@@ -298,13 +316,11 @@ def run_cmd(paths, args):
                         subdirs.remove(dirname)
         else:
             # is file
-            issues.extend(check_path(path,
-                                     os.path.dirname(path),
-                                     args=args,
-                                     force=True) or [])
-            if not args.raw and args.ordered:
-                print_issues(args, issues, os.path.dirname(path))
-
+            new_issues = issues.extend(check_path(path,
+                                                  os.path.dirname(path),
+                                                  args=args,
+                                                  force=True) or [])
+            extend_maybe(issues, new_issues)
         sys.stdout.flush()
 
         # write to std_err so that report on std_out remains unchanged
@@ -312,8 +328,8 @@ def run_cmd(paths, args):
     return 0
 
 
-def get_unilint_parser():
-    parser = ArgumentParser()
+def get_unilint_parser(description=None):
+    parser = ArgumentParser(description=description)
     parser.add_argument('path', nargs='*',
                         help='a folder or filename to check')
     parser.add_argument("-v", "--verbose", dest="verbose", default=False,
@@ -363,6 +379,10 @@ def get_unilint_parser():
 
 
 def evaluate_options(args):
+    """
+    check command line options are valid
+    :returns: args, path
+    """
     if len(args.path) == 0:
         paths = [os.getcwd()]
     else:
@@ -384,9 +404,10 @@ def evaluate_options(args):
 
 
 def unilint_main(argv):
-    prog = argv[0]
+    """
+    CLI entry point
+    """
     args = argv[1:]
-
     parser = get_unilint_parser()
     args = parser.parse_args(args)
 
